#ifndef  _DDWRAPPER_CODE
#define _DDWRAPPER_CODE

#include "DDWrapper.h"
#include "fixed_math.h"
#include <stdlib.h>



/* Must run this proc to fill the RGB16 struct with the information needed to plot a pixel
   To call this, you must have rgb16 defined as a global (unless you want to modify this) variable

   RGB16 rgb16;

return values:
   -1  = bad surface, unable to get description
    0  = unable to obtain bit depth.
    8  = 8  bit depth.
    16 = 16 bit depth.
    24 = 24 bit depth.
    32 = 32 bit depth.
*/
int DDWrapper::SetRGB16(LPDIRECTDRAWSURFACE Surface)
{
   DDSURFACEDESC   ddsd;
   BYTE shiftcount;

   // get a surface despriction
   ddsd.dwSize = sizeof(ddsd);
   ddsd.dwFlags = DDSD_PIXELFORMAT;
   if(Surface->GetSurfaceDesc(&ddsd) != DD_OK)
      return FALSE;

   // Fill in the masking values for extracting colors
   rgb16.Mask.rgbRed   = ddsd.ddpfPixelFormat.dwRBitMask;
   rgb16.Mask.rgbGreen = ddsd.ddpfPixelFormat.dwGBitMask;
   rgb16.Mask.rgbBlue  = ddsd.ddpfPixelFormat.dwBBitMask;

   // get red
   shiftcount = 0;
   while(!(ddsd.ddpfPixelFormat.dwRBitMask & 1))
   {
      ddsd.ddpfPixelFormat.dwRBitMask >>= 1;
      shiftcount++;
   }
   rgb16.depth.rgbRed    = (BYTE)ddsd.ddpfPixelFormat.dwRBitMask;
   rgb16.Position.rgbRed = shiftcount;
   rgb16.Amount.rgbRed   = (ddsd.ddpfPixelFormat.dwRBitMask == 0x1f) ? (BYTE)3 : (BYTE)2;

   // get green
   shiftcount = 0;
   while(!(ddsd.ddpfPixelFormat.dwGBitMask & 1))
   {
      ddsd.ddpfPixelFormat.dwGBitMask >>= 1;
      shiftcount++;
   }
   rgb16.depth.rgbGreen    = (BYTE)ddsd.ddpfPixelFormat.dwGBitMask;
   rgb16.Position.rgbGreen = shiftcount;
   rgb16.Amount.rgbGreen   = (ddsd.ddpfPixelFormat.dwGBitMask == 0x1f) ? (BYTE)3 : (BYTE)2;

   // get Blue
   shiftcount = 0;
   while(!(ddsd.ddpfPixelFormat.dwBBitMask & 1))
   {
      ddsd.ddpfPixelFormat.dwBBitMask >>= 1;
      shiftcount++;
   }
   rgb16.depth.rgbBlue    = (BYTE)ddsd.ddpfPixelFormat.dwBBitMask;
   rgb16.Position.rgbBlue = shiftcount;
   rgb16.Amount.rgbBlue   = (ddsd.ddpfPixelFormat.dwBBitMask == 0x1f) ? (BYTE)3 : (BYTE)2;

   return ddsd.ddpfPixelFormat.dwRGBBitCount;
}

// #####################################################################################

int DDWrapper::Create256Palette(void)
{
    int 				 idx;
    int 				 r, g, b;
    int                  col;

    ape[0].peRed   = 0;
    ape[0].peGreen = 0;
    ape[0].peBlue  = 0;
	ape[0].peFlags = PC_EXPLICIT;

    idx = 1;
    for (int loop = 0; loop < 7; loop++)
    {
		switch (loop)
        {
        	case 0:
            	 r = 255;
            	 for (col = 0; col < 36; col++, idx++)
                 {
                 	ape[idx].peRed   = (BYTE)r;
                    ape[idx].peGreen = 0;
				    ape[idx].peBlue  = 0;
                    ape[idx].peFlags = PC_EXPLICIT;
                    r -= 6;
                 }
                 break;

	        case 1:
            	 g = 255;
            	 for (col = 0; col < 36; col++, idx++)
                 {
                 	ape[idx].peRed   = 0;
                    ape[idx].peGreen = (BYTE)g;
				    ape[idx].peBlue  = 0;
                    ape[idx].peFlags = PC_EXPLICIT;
                    g -= 6;
                 }
                 break;

        	case 2:
               	 b = 255;
            	 for (col = 0; col < 36; col++, idx++)
                 {
                 	ape[idx].peRed   = 0;
                    ape[idx].peGreen = 0;
				    ape[idx].peBlue  = (BYTE)b;
                    ape[idx].peFlags = PC_EXPLICIT;
                    b -= 6;
                 }
                 break;

	        case 3:
            	 r = 255;
               	 g = 255;
            	 for (col = 0; col < 36; col++, idx++)
                 {
                 	ape[idx].peRed   = (BYTE)r;
                    ape[idx].peGreen = (BYTE)g;
				    ape[idx].peBlue  = 0;
                    ape[idx].peFlags = PC_EXPLICIT;
                    r -= 6;
                    g -= 6;
                 }
                 break;

        	case 4:
            	 g = 255;
               	 b = 255;
            	 for (col = 0; col < 36; col++, idx++)
                 {
                 	ape[idx].peRed   = 0;
                    ape[idx].peGreen = (BYTE)g;
				    ape[idx].peBlue  = (BYTE)b;
                    ape[idx].peFlags = PC_EXPLICIT;
                    b -= 6;
                    g -= 6;
                 }
                 break;

	        case 5:
            	 r = 255;
               	 b = 255;
            	 for (col = 0; col < 37; col++, idx++)
                 {
                 	ape[idx].peRed   = (BYTE)r;
                    ape[idx].peGreen = 0;
				    ape[idx].peBlue  = (BYTE)b;
                    ape[idx].peFlags = PC_EXPLICIT;
                    b -= 6;
                    r -= 6;
                 }
                 break;

        	case 6:
	        default:
            	 r = 255;
                 g = 255;
               	 b = 255;
            	 for (col = 0; col < 38; col++, idx++)
                 {
                 	ape[idx].peRed   = (BYTE)r;
                    ape[idx].peGreen = (BYTE)g;
				    ape[idx].peBlue  = (BYTE)b;
                    ape[idx].peFlags = PC_EXPLICIT;
                    b -= 5;
                    g -= 5;
                    r -= 5;
                 }
		}
    }

    if (pDD->CreatePalette(DDPCAPS_ALLOW256 | DDPCAPS_8BIT, ape, &pDDPalette, NULL) != DD_OK)
		return FALSE;

    // this sets the palette for the primary surface
    if (pDDSPrimary->SetPalette(pDDPalette) !=  DD_OK)
		return FALSE;

    return TRUE;
}

// #####################################################################################

/* Return values
    0  = successful initialisation,
   -1  = bad hwnd.
   -2  = unable to init directdraw object.
   -3  = unable to set full screen mode.
   -4  = unable to set window mode.
   -5  = unable to create surface.
   -6  = unable to determine pixel format.
   -7  = unable to create clipper rect.
   -8  = unable to assign window handle to clipper.
   -9  = unable to assign clipper to direct draw object.
*/
int DDWrapper::InitDDraw(HWND hwnd, BOOL fullScreen, int width, int height, int depth)
{
    DDSURFACEDESC ddsd; // A structure to describe the surface we want
    HRESULT 	  hr;

    // Check parameters
    if (hwnd == NULL)
        return -1;
    else
        ddWnd = hwnd;

    bFullScreen = fullScreen;

    CleanUp();

    // --------- Create the DirectDraw object ---------

    // The first NULL means use the active display driver
    // The last parameter must be NULL
    hr = DirectDrawCreate(NULL, &pDD, NULL);

    if (hr != DD_OK)
    {
    	dDrawActive = false;
        return -2;
    }

    // ------------ Set display mode ------------

    if (bFullScreen != FALSE)
    {
    	// Set the "cooperative level" so we can use full-screen mode
        hr = pDD->SetCooperativeLevel(hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN |
      				      DDSCL_NOWINDOWCHANGES);
        if (hr != DD_OK)
        {
            CleanUp();
            return -3;
    	}

        hr = pDD->SetDisplayMode(width, height, depth);

        if (hr != DD_OK)
        {
            CleanUp();
            return -3;
        }

    }
    else
    {
    	// Set DDSCL_NORMAL to use windowed mode
        hr = pDD->SetCooperativeLevel(hwnd, DDSCL_NORMAL);
        if (hr != DD_OK)
        {
            CleanUp();
            return -4;
        }
    }

    // -------------- Create Surface ----------

    // This clears all fields of the DDSURFACEDESC structure to 0
    memset(&ddsd, 0, sizeof(ddsd));

    // The first parameter of the structure must contain the
    // size of the structure.
    ddsd.dwSize = sizeof(ddsd);

    // The dwFlags paramater tell DirectDraw which DDSURFACEDESC
    // fields will contain valid values
    ddsd.dwFlags        = DDSD_CAPS;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

    // Create the primary surface
    hr = pDD->CreateSurface(&ddsd, &pDDSPrimary, NULL);
    if (hr != DD_OK)
    {
		CleanUp();
        return -5;
    }

    // ----------- Create Clipper Rect ------------

    // Create the clipper using the DirectDraw object
    hr = pDD->CreateClipper(0, &pDDClipper, NULL);

    if (hr != DD_OK)
    {
        CleanUp();
        return -7;
    }

    // Assign your window's HWND to the clipper
    hr = pDDClipper->SetHWnd(0, hwnd);
    if (hr != DD_OK)
    {
       	CleanUp();
        return -8;
    }

    // Attach the clipper to the primary surface
    hr = pDDSPrimary->SetClipper(pDDClipper);
    if (hr != DD_OK)
    {
        CleanUp();
        return -9;
    }

    // ---------- determine bit depth ------------

    switch ((bitDepth = depth))
    {
    	case  8:
    		if (Create256Palette () == FALSE)
            	return -6;
	        break;

    	case 16:
		    if (SetRGB16(pDDSPrimary) <= 0)
    		{
    			CleanUp();
		       	return -6;
    		}
    }

    dDrawActive = true;

    return 0;
}

// #####################################################################################

void DDWrapper::PlotPixel (int x, int y, COLORREF col)
{
    HRESULT 	  rval;
    DDSURFACEDESC ddsd;
   	static int    idx;

    if ((x < 0) || (x >= maxX))
    	return;

    if ((y < 0) || (y >= maxY))
    	return;


    if (pDDSPrimary == NULL)
    	return;

    // *** get surface description and lock the surface to draw pixel ***
    memset (&ddsd, 0, sizeof (DDSURFACEDESC));
    ddsd.dwSize = sizeof (DDSURFACEDESC);

    rval = pDDSPrimary -> Lock(NULL, &ddsd, DDLOCK_WAIT, NULL);
    if (rval != DD_OK)
    	return;

    switch (bitDepth)
    {
    	case  8:   // 256 colour mode
			((BYTE*)ddsd.lpSurface)[ y * ddsd.lPitch + x] = (BYTE)col;
        	break;

    	case 16:   // 16 bit mode
  	    	((unsigned short*)ddsd.lpSurface)[(y * (ddsd.lPitch >> 1) + x)] =
            (unsigned short)RGB16 (GetRValue(col), GetGValue(col), GetBValue(col));
            break;

    	case 24:   // 24 bit mode
           	idx = (y * ddsd.lPitch + (x * 3));
        	((BYTE*)ddsd.lpSurface)[idx]   = GetRValue(col);
           	((BYTE*)ddsd.lpSurface)[idx+1] = GetGValue(col);
           	((BYTE*)ddsd.lpSurface)[idx+2] = GetBValue(col);
            break;
    }

    pDDSPrimary-> Unlock (NULL);
}

// #####################################################################################

BOOL DDWrapper::LockPrimarySurface (void)
{
    if ((pDDSPrimary == NULL) || (dDrawActive == FALSE))
    	return FALSE;

    // *** get surface description and lock the surface to draw pixel ***
    memset (&ddsd, 0, sizeof (DDSURFACEDESC));
    ddsd.dwSize = sizeof (DDSURFACEDESC);

    if (pDDSPrimary -> Lock(NULL, &ddsd, DDLOCK_WAIT, NULL) != DD_OK)
    	return FALSE;

    return TRUE;
}

// #####################################################################################

void DDWrapper::FastPlotPixel (int x, int y, COLORREF col)
{
	static int idx;

    if ((x < 0) || (x >= maxX))
    	return;

    if ((y < 0) || (y >= maxY))
    	return;

    switch (bitDepth)
    {
    	case 8:	   // 256 colour mode
			((BYTE*)ddsd.lpSurface)[y * ddsd.lPitch + x] = (BYTE)col;
            return;

    	case 16:   // 16 bit mode
	   	    ((unsigned short*)ddsd.lpSurface)[(y * (ddsd.lPitch >> 1) + x)] =
            	(unsigned short)RGB16 (GetRValue(col), GetGValue(col), GetBValue(col));
            return;

    	case 24:   // 24 bit mode
        	idx = (y * ddsd.lPitch + (x * 3));
        	((BYTE*)ddsd.lpSurface)[idx]   = GetRValue(col);
           	((BYTE*)ddsd.lpSurface)[idx+1] = GetGValue(col);
           	((BYTE*)ddsd.lpSurface)[idx+2] = GetBValue(col);
            return;
    }
}

// #####################################################################################

void DDWrapper::UnlockPrimarySurface (void)
{
    if ((pDDSPrimary == NULL) || (dDrawActive == FALSE))
    	return;

    pDDSPrimary-> Unlock (NULL);
}

#endif // _DDWRAPPER_CODE
