//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include <time.h>
#include <stdlib.h>
#include <ddraw.h>
#include <mmsystem.h>

#include "Prefs.h"
#include "SSFrame.h"
#include "Fclasses.h"
#include "fixed_math.h"

//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TSSFrameO *SSFrameO;

// Current system screen size.
int MAXX, MAXY;

//---------------------------------------------------------------------------

__fastcall TSSFrameO::TSSFrameO(TComponent* Owner)	: TForm(Owner)
{
    randomize     ();
    BuildTables   ();
    LoadPrefsInfo ();

    Cursor        = crNone;
    pSkyRockets   = NULL;
    pFlowerPots   = NULL;
    pMultiRockets = NULL;
    threadID      = NULL;

    // Allocate memory
    if (prefs.srNum > 0)
    {
		if ((pSkyRockets = new SkyRocket [prefs.srNum]) == NULL)
        {
			MessageBox (Handle, "*** Out Of Memory ***",
                                "Unable to start pyro", MB_ICONINFORMATION | MB_OK);
			BreakScreenSaver (true);
        }
    }

    if (prefs.ssrNum > 0)
    {
		if ((pMultiRockets = new MultiRocket [prefs.ssrNum]) == NULL)
        {
			MessageBox (Handle, "*** Out Of Memory ***",
                        "Unable to start pyro", MB_ICONINFORMATION | MB_OK);
			BreakScreenSaver (true);
        }
    }

    if (prefs.pNum > 0)
    {
		if ((pFlowerPots = new FlowerPot [prefs.pNum]) == NULL)
        {
			MessageBox (Handle, "*** Out Of Memory ***",
                        "Unable to start pyro", MB_ICONINFORMATION | MB_OK);
			BreakScreenSaver (true);
        }
    }

    Top = Left = 0;
   	ddObj.maxX = MAXX = ClientWidth  = prefs.xRes;
    ddObj.maxY = MAXY = ClientHeight = prefs.yRes;
	PixelSize  = prefs.pixelSize;
	ExplodeSnd = prefs.explodeSnd;
    switch (ddObj.InitDDraw(Handle, true, ddObj.maxX, ddObj.maxY, prefs.bitDepth))
    {
	   	case 0:
	       	break;

    	default:
			MessageBox (Handle, "*** Unable to obtain defined screen resolution"
                        " in user preferences ***", "Unable to start pyro",
                        MB_ICONINFORMATION | MB_OK);
	     	BreakScreenSaver (true);
    }

    // Create a separate thread to handle updating of fireworks
    threadExitSemaphore = false;
    threadDieSemaphore  = false;
    CreateThread (NULL, 0, ThreadFunc, (LPDWORD) this, 0, &threadID);

    if (threadID == NULL)
    	BreakScreenSaver (true);
}

//---------------------------------------------------------------------------

void __fastcall TSSFrameO::FormMouseMove(TObject *Sender, TShiftState Shift,
	int X, int Y)
{
	static int    mouseMove = 0;
    static time_t lastTime  = -1;
    static time_t thisTime;

    time (&thisTime);

    // movements occured in lest than 3 seconds apart.
    if ((lastTime < 0) ||
        (thisTime-lastTime) < 3)
    {
        mouseMove++;
        if (mouseMove >= 5)
       		BreakScreenSaver (true);
    }
    else
    	mouseMove = 0;

    lastTime = thisTime;
}

//---------------------------------------------------------------------------

void __fastcall TSSFrameO::FormKeyPress(TObject *Sender, char &Key)
{
	BreakScreenSaver (true);
}

//---------------------------------------------------------------------------

void  TSSFrameO::BreakScreenSaver (BOOL dieNow)
{
	if (dieNow == FALSE)
    {
	    threadExitSemaphore = true;

	    while (threadDieSemaphore == false)
    		Sleep (30);
    }

	Application -> Terminate();
}

//---------------------------------------------------------------------------

// Pic one of the primary colours
COLORREF TSSFrameO::PicColour (void)
{
	switch (prefs.bitDepth)
    {
    	case 8:
        	return (COLORREF) randomTab (255)+1; /* 1 - 255 */

    	case 16:
        case 24:
    	// select primary colour
	    switch (randomTab(7))
    	{
        	case 0:
            	 return RGB (255, 1, 1);

	        case 1:
    	         return RGB (1, 255, 1);

        	case 2:
            	 return RGB (1, 1, 255);

	        case 3:
    	         return RGB (255, 255, 1);

        	case 4:
            	 return RGB (1, 255, 255);

	        case 5:
    	         return RGB (255, 1, 255);

        	case 6:
	        default:
    	         return RGB (255, 255, 255);
    	}
    }

    return RGB (255,255,255);
}

//---------------------------------------------------------------------------

void TSSFrameO::LoadPrefsInfo (void)
{
    const int     BuffSize = 128;
    char          profileBuff[BuffSize];
    HKEY          hWindowInfo = NULL;
    unsigned long type        = REG_SZ;
    unsigned long fucked      = BuffSize;
    LONG          result      = RegOpenKey (HKEY_CURRENT_USER,
                                            (LPCTSTR)"software\\De Toni Ware",
                                            &hWindowInfo);

    if (result != 0)
        strcpy (profileBuff, DefaultProfile);
	else
    {
    	result = RegQueryValueEx(hWindowInfo, (LPTSTR)"Pyro Info", NULL, &type,
        	                     (LPBYTE)profileBuff, &fucked);

        // Unable to obtain pyro profile, then use defaults.
        if (result != 0)
        	strcpy (profileBuff, DefaultProfile);

        RegCloseKey(hWindowInfo);
    }

    // Get info window form
    sscanf (profileBuff, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
    &prefs.explodeSnd,
    &prefs.pixelSize,
    &prefs.srNum,  &prefs.srExdNum,  &prefs.srTrailLen, &prefs.srCyc,
    &prefs.ssrNum, &prefs.ssrPodNum, &prefs.ssrExdNum,  &prefs.ssrTrailLen, &prefs.ssrCyc,
    &prefs.pNum,   &prefs.pExdNum,   &prefs.pTrailLen,  &prefs.pCyc,        &prefs.xRes,
    &prefs.yRes,   &prefs.bitDepth);


}

//---------------------------------------------------------------------------

void __fastcall TSSFrameO::RefreshTimer(TObject *Sender)
{
	int idx;

	if (ddObj.LockPrimarySurface() == FALSE)
    	return;

    for (idx = 0; idx < prefs.srNum; idx++)
    {
		if (pSkyRockets[idx].move(ddObj) == FALSE)
	    {
    		pSkyRockets[idx].init (prefs.srExdNum,             // N.O of flares
        	               ClientWidth >> 1, ClientHeight-1,   // x, y start location
        			 	   ((randomTab (15)-7) << SHIFT),      // Horz thrust
                	       ((randomTab (15)+10) << SHIFT),     // Vert thrust
                    	   randomTab(prefs.srCyc),             // Life counter
	                       ClientHeight,                       // Ground
    	                   prefs.srTrailLen,                   // Trailer Length
        	               PicColour()                         // colour
            	           );
        }
    }

	for (idx = 0; idx < prefs.pNum; idx++)
    {
       	if (pFlowerPots[idx].move(ddObj) == FALSE)
	    {
    		pFlowerPots[idx].init (prefs.pExdNum,                               // N.O of flares
                                   randomTab(MAXX >> 1) + (MAXX >> 2), MAXY,       // x, y start location
                                   PicColour (),                                // colour
                                   randomTab(prefs.pCyc >> 1) + (prefs.pCyc >> 1), // Life counter
                                   prefs.pTrailLen                              // Trailer Length
                                  );
        }

    }

    for (idx = 0; idx < prefs.ssrNum; idx++)
    {
    	if (pMultiRockets[idx].move(ddObj) == FALSE)
	    {
	    	pMultiRockets[idx].init (prefs.ssrPodNum, prefs.ssrExdNum,   // N.O of flares
    	                             ClientWidth >> 1, ClientHeight-1,   // x, y start location
        			           	     ((randomTab (15)-7) << SHIFT),      // Horz thrust
            	    	             ((randomTab (15)+10) << SHIFT),     // Vert thrust
                	                 randomTab(prefs.ssrCyc),            // Life counter
	                	             ClientHeight,                       // Ground
	    	                         prefs.ssrTrailLen,                  // Trailer Length
    	    	                     PicColour()                         // colour
        	    	                );
	    }
    }

    ddObj.UnlockPrimarySurface();

/**************************************************************************************

    char buff[1024];
    TPen tp();
    time_t t1, t2;

    t1 = time (NULL);

    for (int y = 0; y < ClientHeight; y++)
    {
    	for (int x = 0; x < ClientWidth; x++)
        {
             ddObj.PlotPixel (x, y, RGB(x % 255, y % 255, x+y % 255));
        }
    }

   	t2 = time(NULL);
	sprintf (buff, "Time taken = %d", t2 - t1);
	MessageBox (this -> Handle, buff, "Auto Lock", MB_OK);

    // #####################################################################

	if (ddObj.LockPrimarySurface() == FALSE)
    {
    	MessageBox (this -> Handle, "Unable to lock surface", "", MB_OK);
        return;
    }

    t1 = time (NULL);
    for (int y = 0; y < ClientHeight; y++)
    {
    	for (int x = 0; x < ClientWidth; x++)
        {
             ddObj.FastPlotPixel (x, y, RGB(0,255,0));
        }
    }

   	t2 = time(NULL);
    ddObj.UnlockPrimarySurface();
	sprintf (buff, "Time taken = %d", t2 - t1);
	MessageBox (this -> Handle, buff, "Fast Plot", MB_OK);

    // #####################################################################

    t1 = time (NULL);
    for (int y = 0; y < ClientHeight; y++)
    {
    	for (int x = 0; x < ClientWidth; x++)
        {
       		Canvas -> Pixels [x][y] = (TColor) RGB (0, 0, 255);
        }
    }

   	t2 = time(NULL);
	sprintf (buff, "Time taken = %d", t2 - t1);
	MessageBox (this -> Handle, buff, "C++ Builder", MB_OK);

    // #####################################################################

    t1 = time (NULL);
    for (int y = 0; y < ClientHeight; y++)
    {
    	for (int x = 0; x < ClientWidth; x++)
        {
       		SetPixel (Canvas -> Handle, x, y, RGB (0, 255, 0));
        }
    }

   	t2 = time(NULL);
	sprintf (buff, "Time taken = %d", t2 - t1);
	MessageBox (this -> Handle, buff, "Win API", MB_OK);

    // #####################################################################

    HDC dc = GetDC (Handle);

    t1 = time (NULL);
    for (int y = 0; y < ClientHeight; y++)
    {
    	for (int x = 0; x < ClientWidth; x++)
        {
       		SetPixel (dc, x, y, RGB (255, 255, 0));
        }
    }

    ReleaseDC (Handle, dc);

   	t2 = time(NULL);
	sprintf (buff, "Time taken = %d", t2 - t1);
	MessageBox (this -> Handle, buff, "Win API HDC", MB_OK);


	for (int y = 0; y < 200000; y++)
	{
		Canvas -> Pen ->Color = (TColor)RGB (random(256), random(256), random(256));
		Canvas -> MoveTo (random(ClientWidth), random(ClientHeight));
		Canvas -> LineTo (random(ClientWidth), random(ClientHeight));
	}

	for (int pixsPerFrame = 0; pixsPerFrame < 200000; pixsPerFrame++)
	{


		Canvas -> Pixels [random(this -> ClientWidth)]
						 [random(this -> ClientHeight)] =
						 (TColor) RGB
						 (random(256), random(256), random(256));



		SetPixel (Canvas -> Handle,
				  random(this -> ClientWidth),
				  random(this -> ClientHeight),
				  RGB (random(256), random(256), random(256)));



	}
***************************************************************************************/

}

//---------------------------------------------------------------------------

void __fastcall TSSFrameO::FormPaint(TObject *Sender)
{
	RefreshTimer(this);
}

//---------------------------------------------------------------------------

void __fastcall TSSFrameO::FormDestroy(TObject *Sender)
{
	if (pSkyRockets != NULL)
	{
    	delete[] pSkyRockets;
        pSkyRockets = NULL;
    }

    if (pFlowerPots != NULL)
    {
		delete[] pFlowerPots;
        pFlowerPots = NULL;
    }

    if (pMultiRockets != NULL)
    {
    	delete[] pMultiRockets;
        pMultiRockets = NULL;
    }

}

//---------------------------------------------------------------------------

DWORD WINAPI ThreadFunc (LPVOID pData)
{
	int        idx;
    TSSFrameO* pSSObj = (TSSFrameO*) pData;
    DWORD      startTime, endTime;

    while (pSSObj->threadExitSemaphore == false)
    {

    	startTime = GetTickCount ();

		if (pSSObj->ddObj.LockPrimarySurface() == FALSE)
    		return -1;

	    for (idx = 0; idx < pSSObj->prefs.srNum; idx++)
    	{
			if (pSSObj->pSkyRockets[idx].move(pSSObj->ddObj) == FALSE)
		    {
            	// Stager when sky rockets are launched for a better effect
            	if (randomTab(60) == 30)
                {
    				pSSObj->pSkyRockets[idx].init (
                    	pSSObj->prefs.srExdNum,                             // N.O of flares
	                    pSSObj->ClientWidth >> 1, pSSObj->ClientHeight-1,   // x, y start location
    	    			((randomTab (15)-7) << SHIFT),                      // Horz thrust
        	        	((randomTab (15)+10) << SHIFT),                     // Vert thrust
            	        randomTab(pSSObj->prefs.srCyc),                     // Life counter
	            	    pSSObj->ClientHeight,                               // Ground
	    	        	pSSObj->prefs.srTrailLen,                           // Trailer Length
	    	    	    pSSObj->PicColour()                                 // colour
    	    	    	                          );

        	    	if (ExplodeSnd != 0)
    	    	    {
	            		pSSObj->ddObj.UnlockPrimarySurface();
			    		PlaySound ("fire.wav", NULL, SND_ASYNC);
    	   	            pSSObj->ddObj.LockPrimarySurface();
        		    }
                }
	        }
    	}

		for (idx = 0; idx < pSSObj->prefs.pNum; idx++)
    	{
       		if (pSSObj->pFlowerPots[idx].move(pSSObj->ddObj) == FALSE)
		    {
    			pSSObj->pFlowerPots[idx].init (
                    pSSObj->prefs.pExdNum,                                          // N.O of flares
        	        randomTab(MAXX >> 1) + (MAXX >> 2), MAXY,                       // x, y start location
            	    pSSObj->PicColour (),                                           // colour
                	randomTab(pSSObj->prefs.pCyc >> 1) + (pSSObj->prefs.pCyc >> 1), // Life counter
                    pSSObj->prefs.pTrailLen                                         // Trailer Length
                        	                  );
	        }
    	}

	    for (idx = 0; idx < pSSObj->prefs.ssrNum; idx++)
    	{
    		if (pSSObj->pMultiRockets[idx].move(pSSObj->ddObj) == FALSE)
		    {
            	// Stager when sky rockets are launched for a better effect
            	if (randomTab(60) == 30)
                {
			    	pSSObj->pMultiRockets[idx].init (
    	                pSSObj->prefs.ssrPodNum, pSSObj->prefs.ssrExdNum,   // N.O of flares
    			        pSSObj->ClientWidth >> 1, pSSObj->ClientHeight-1,   // x, y start location
        				((randomTab (15)-7) << SHIFT),                      // Horz thrust
            			((randomTab (15)+10) << SHIFT),                     // Vert thrust
	                	randomTab(pSSObj->prefs.ssrCyc),                    // Life counter
		                pSSObj->ClientHeight,                               // Ground
	    		        pSSObj->prefs.ssrTrailLen,                          // Trailer Length
    	    		    pSSObj->PicColour()                                 // colour
        	    		                	        );

	              	if (ExplodeSnd != 0)
    		        {
	    	        	pSSObj->ddObj.UnlockPrimarySurface();
				    	PlaySound ("fire.wav", NULL, SND_ASYNC);
       	        	    pSSObj->ddObj.LockPrimarySurface();
	        	    }
                }
		    }
    	}

	    pSSObj->ddObj.UnlockPrimarySurface();

        endTime = GetTickCount ();

        if ((endTime-startTime) < 15)
     		Sleep(15 - (endTime-startTime));
	}

    pSSObj->threadDieSemaphore = true;
    return 0;
}


