// fire_shell.java

#ifndef _fire_shell
#define _file_shell

#include <windows.h>
#include <math.h>
#include "sphere.h"
#include "fixed_math.h"

class Sphere
{
	public:
    double *px, *py, *pz;
    int    stars;
    double theta, phy, cphy, sphy;
    double pi;
    int    ratio;
    int    i, j;

    Sphere ()
    {
	    pi    = 3.141592;
    	ratio = 3;
    	px = py = pz = NULL;
    }

    Sphere (int n)
    {
    	Sphere();
        init(n);
    }

    void init (int n)
    {
        int row = 2 * n-1;
        int k   = 0;
        int l;

        // Calculate number of stars required.
        for(i = 1; i <= row; i++)
            k += (int) (n * ratio * sin((pi*i) / (row+1)));

        // Allocate memory.
        stars = k;
        px    = new double [stars];
        py    = new double [stars];
        pz    = new double [stars];

        for(i = 1, k = 0; i <= row; i++)
        {
            phy  = pi*(double)(i)/(double)(row+1);
            cphy = cos(phy);
            sphy = sin(phy);
            l    = (int) (n * ratio * sin((pi*i) / (row+1)));

            for (j=0;j<l;j++)
            {
                theta = 2.0 * pi * (double)(j) / (double)l;
                px[k] = sphy * cos(theta) * n / 6;
                py[k] = sphy * sin(theta) * n / 6;
                pz[k] = cphy * n / 6;
                k++;
            }
        }
    }

	~Sphere ()
    {
		if (px != NULL)
			delete px;

        if (py != NULL)
        	delete py;

        if (pz != NULL)
        	delete pz;
    }
};

class fire_shell
{
	public:
	double grav;
	double *vx, *vy;
	double *ox, *oy, *oz;
	double length;

	int	ranlen;
	int baselen;
	int width, height;
	int cx, cy;
	int stars;
	int i, j, r;
	int life, start, end, runlength, runlength2;


	fire_shell (int px, int py, int ir, int it)
	{
		ranlen  = 3,
		baselen = 40;
		start = it;
		end   = it + ir;
		life  = ir + 100;
		cx    = px;
		cy    = py;
		grav  = 0.006;
		r     = 0;
	}

	fire_shell(int px, int py, int ir, int it, Sphere* pSphere)
	{
		stars     = pSphere -> stars;
		vx        = new double [stars];
		vy        = new double [stars];

		ox        = pSphere -> px;
		oy        = pSphere -> py;
		oz        = pSphere -> pz;
		runlength = runlength2 = 1;	// dummy
		start     = it;
		end       = ir + it;
		life      = ir + 100;       // dummy
		grav      = 0.006;          // dummy
		cx        = px;
		cy        = py;
		r         = 0;

		RotateSet();
	}

    void RotateSet ()
    {
  	    double delta,  phy;
		double cdelta, sdelta;
		double cphy,   sphy;
		double x,      y,     z;
		int    rand;

        // Get random amounts for DELTA, and PHY
		delta  = randomTab(360);
		phy    = randomTab(360);

		// Get Sine and Co-Sine values for DELTA
		cdelta = cos(delta);
		sdelta = sin(delta);

		// Get Sine and Co-Sine values for PHY
		cphy   = cos(phy);
		sphy   = sin(phy);

		for(j = 0; j < stars; j++)
		{
			rand   = randomTab(ranlen);
			length = (double) (baselen + rand) / (double)baselen;
			x      = ox[j];
			y      = oy[j]  * cdelta - oz[j] * sdelta;
			z      = oy[j]  * sdelta + oz[j] * cdelta;
			vx[j]  = length * (x * cphy - z * sphy);
			vy[j]  = length * y;
	    }
    }

    BOOL proceed (void)
    {
	    if (r < life)
	    {
		    r++;
		    return(false);
	    }
	    else
	    {
		    init_round();
		    return(true);
	    }
    }

    void drawStars (COLORREF col)
    {
	    int dr, ot;
	    int x,  y;

	    if (r > start && runlength2 > 0)
	    {
//            g.setColor(col1);
		    dr = r - start;

		    // Update the fire work.
            for (i = 0; i < stars; i++)
            {
			    x = (int) (dr * vx[i]) + cx;
			    y = (int) ((vy[i] + grav * dr) * dr) + cy;

//                g.drawLine(x, y, x, y);
		    }

		    // Delete old pixels....
		    if (runlength2 > 0 && r - runlength2 > start)
		    {
//			    g.setColor(Color.black);
			    dr = r - runlength2 - start;

			    for (i = 0; i < stars; i++)
			    {
				    x = (int)  (dr * vx[i]) + cx;
				    y = (int) ((vy[i] + grav * dr) * dr) + cy;

//				    g.drawLine(x, y, x, y);
			    }

			    if (runlength2 > 0 && r > end)
			    {
				    runlength2--;
				    dr = r - runlength2 - start;

				    for (i = 0; i < stars; i++)
				    {
					    x = (int)  (dr * vx[i]) + cx;
					    y = (int) ((vy[i] + grav * dr) * dr) + cy;
//					    g.drawLine(x, y, x, y);
				    }
			    }
		    }
	    }
    }

    void setGravity(double g)
    {
	    grav = g;
    }


    void setRunLength(int rl)
    {
      runlength = runlength2 = rl;
    }

    void setLifeTime(int lt)
    {
      life = lt;;
    }


    void init_round()
    {
	    runlength2 = runlength;
	    r = 0;
    }
};

#endif
